#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.shortcuts import render, get_object_or_404
import json
import sys
from django.contrib import messages
from django.views.decorators.csrf import csrf_exempt
from ..decorators import (httpauth_login_required, ip_write_verification_required,
                          kickout_400, kickout_404, kickout_500)
from django.http import HttpResponse, HttpResponseRedirect
from collections import OrderedDict
from ..mongoutils import write_mongo, checkObjectId
from jsonschema import validate
from django.urls import reverse
from jsonschema.exceptions import ValidationError
from .models import WriteAPIHTTPAuth, WriteAPIIP, WriteAPIOAuth2
from .forms import (WriteAPIHTTPAuthForm, WriteAPIHTTPAuthDeleteForm,
                    WriteAPIIPDeleteForm, WriteAPIIPForm)
from .forms import WriteAPIOAuth2Form, WriteAPIOAuth2DeleteForm
from django.utils.translation import ugettext_lazy as _


def write_to_collection(request, slug, model):
    try:
        wapi = model.objects.get(slug=slug)
    except model.DoesNotExist:
        return kickout_404(
            "The API was not not found. Perhaps you need to define it?")
    if request.method == 'GET':
        return HttpResponse(
            json.dumps(wapi.http_get_response(), indent=4),
            content_type="application/json")

    # ----------------------------------------------------
    if request.method not in wapi.http_methods():
        msg = "The HTTP method %s is not allowed." % (request.method)
        return kickout_400(msg)

    if request.method in ('POST', 'PUT'):

        # Check if request body is JSON ------------------------
        try:
            j = json.loads(request.body.decode(),
                           object_pairs_hook=OrderedDict)
            if not isinstance(j, type(OrderedDict())):
                kickout_400(
                    "The request body did not contain a JSON object i.e. {}.")
        except:
            print(str(sys.exc_info()))
            return kickout_400("The request body did not contain valid JSON.")

        # Make sure that an update (PUT) contains an id or _id
        if request.method == "PUT":
            if "id" not in j.keys() and "_id" not in j.keys():
                return kickout_400("The PUT update request did not contain an id or _id.")
        # Check that id is valid:
        if j.get("id", ""):
            if not checkObjectId(j.get("id", "")):
                return kickout_400("The id was not a valid ObjectId. It must be a 12-byte input or a 24-character hex string.")

        # check json_schema is valid
        try:
            json_schema = json.loads(
                wapi.json_schema, object_pairs_hook=OrderedDict)

        except:
            print(str(sys.exc_info()))
            return kickout_500(
                "The JSON Schema on the server did not contain valid JSON.")

        # Check jsonschema
        if json_schema:
            try:
                validate(j, json_schema)
            except ValidationError:
                msg = "JSON Schema conformance error. %s" % (
                    str(sys.exc_info()[1][0]))
                return kickout_400(msg)
        # write_to_mongo

        if request.method == "POST":
            response = write_mongo(j, wapi.database_name, wapi.collection_name)
        if request.method == "PUT":
            response = write_mongo(j, wapi.database_name,
                                   wapi.collection_name, update=True)
        return HttpResponse(json.dumps(response, indent=4),
                            content_type="application/json")


@csrf_exempt
@httpauth_login_required
def write_to_collection_httpauth(request, slug):
    print("REFACTOR!!")
    return write_to_collection(request, slug, WriteAPIHTTPAuth)


@csrf_exempt
@ip_write_verification_required
def write_to_collection_ip_auth(request, slug):
    print("REFACTOR!!")
    return write_to_collection(request, slug, WriteAPIIP)


def browse_ip_write_apis(request, database_name=None, collection_name=None):
    """Deprecated"""
    name = "Write APIs Using IP-based Authentication"
    if database_name and collection_name:
        wapis = WriteAPIIP.objects.filter(
            database_name=database_name,
            collection_name=collection_name)
    else:
        wapis = WriteAPIIP.objects.all()
    context = {'name': name, 'wapis': wapis,
               'database_name': database_name,
               'collection_name': collection_name}
    return render(
        request,
        'djmongo/console/browse-ip-write-apis.html',
        context)


def browse_httpauth_write_apis(
        request,
        database_name=None,
        collection_name=None):
    """Deprecated"""
    name = "Write APIs Using HTTPAuth Authentication"
    if database_name and collection_name:
        wapis = WriteAPIHTTPAuth.objects.filter(
            database_name=database_name,
            collection_name=collection_name)
    else:
        wapis = WriteAPIHTTPAuth.objects.all()
    context = {'name': name, 'wapis': wapis,
               'database_name': database_name,
               'collection_name': collection_name}
    return render(
        request,
        'djmongo/console/browse-httpauth-write-apis.html',
        context)


def create_httpauth_write_api(
        request,
        database_name=None,
        collection_name=None):
    name = _("Create an HTTP Auth Write API")
    if request.method == 'POST':
        form = WriteAPIHTTPAuthForm(request.POST)
        if form.is_valid():
            a = form.save()
            msg = _('The HTTP Auth write API for %s was created.') % (a.slug)
            messages.success(request, msg)

            return HttpResponseRedirect(reverse('djmongo_show_apis',
                                                args=(a.database_name,
                                                      a.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            context = {'form': form, 'name': name}
            return render(
                request,
                'djmongo/console/generic/bootstrapform.html',
                context)

    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}
    context = {'name': name,
               'form': WriteAPIHTTPAuthForm(initial=idata)
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context,
    )


def create_ip_write_api(request, database_name=None, collection_name=None):
    name = _("Create an IP-based Write API")
    if request.method == 'POST':
        form = WriteAPIIPForm(request.POST)
        if form.is_valid():
            a = form.save()
            msg = _('The IP-based write API for %s was created.') % (a.slug)
            messages.success(request, msg)
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(a.database_name,
                              a.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            context = {'form': form, 'name': name}
            return render(
                request,
                'djmongo/console/generic/bootstrapform.html',
                context)

    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}
    context = {'name': name,
               'form': WriteAPIIPForm(initial=idata)
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context,
    )


def edit_httpauth_write_api(request, slug):
    a = get_object_or_404(WriteAPIHTTPAuth, slug=slug)
    name = _("Edit HTTP Auth Write API")
    if request.method == 'POST':
        form = WriteAPIHTTPAuthForm(request.POST, instance=a)
        if form.is_valid():
            a = form.save()
            msg = _('The HTTP Auth API for %s was updated.') % (slug)
            messages.success(request, msg)
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(a.database_name,
                              a.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            context = {'form': form, 'name': name}
            return render(
                request,
                'djmongo/console/generic/bootstrapform.html',
                context)
    # this is a GET
    context = {'name': name,
               'form': WriteAPIHTTPAuthForm(instance=a)
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context,
    )


def edit_ip_write_api(request, slug):
    a = get_object_or_404(WriteAPIIP, slug=slug)
    name = _("Edit IP-based Write API")
    if request.method == 'POST':
        form = WriteAPIIPForm(request.POST, instance=a)
        if form.is_valid():
            a = form.save()
            msg = _('The IP-based API for %s was updated.') % (slug)
            messages.success(request, msg)
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(a.database_name,
                              a.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            context = {'form': form, 'name': name}
            return render(
                request,
                'djmongo/console/generic/bootstrapform.html',
                context)

    # this is a GET
    context = {'name': name,
               'form': WriteAPIIPForm(instance=a)
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context,
    )


def delete_httpauth_write_api(request, slug):
    name = _("Delete HTTP Auth Write API")
    if request.method == 'POST':
        form = WriteAPIHTTPAuthDeleteForm(request.POST)
        if form.is_valid():
            a = WriteAPIHTTPAuth.objects.get(slug=slug)
            a.delete()
            msg = _('The HTTP Auth API for %s was deleted.') % (slug)
            messages.success(request, msg)
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(a.database_name,
                              a.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            context = {'form': form, 'name': name}
            return render(
                request,
                'djmongo/console/generic/bootstrapform.html',
                context)

    # this is a GET
    context = {'name': name,
               'form': WriteAPIHTTPAuthDeleteForm()
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context,
    )


def delete_ip_write_api(request, slug):
    name = _("Delete IP-based Write API")
    if request.method == 'POST':
        form = WriteAPIIPDeleteForm(request.POST)
        if form.is_valid():
            a = WriteAPIIP.objects.get(slug=slug)
            a.delete()
            msg = _('The IP-based API for %s was deleted.') % (slug)
            messages.success(request, msg)
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(a.database_name,
                              a.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            context = {'form': form, 'name': name}
            return render(
                request,
                'djmongo/console/generic/bootstrapform.html',
                context)

    # this is a GET
    context = {'name': name,
               'form': WriteAPIIPDeleteForm()
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context,
    )


def create_oauth2_write_api(
        request,
        database_name=None,
        collection_name=None):
    name = _("Create an OAuth2 Write API")
    if request.method == 'POST':
        form = WriteAPIOAuth2Form(request.POST)
        if form.is_valid():
            a = form.save(commit=False)
            a.created_by = request.user
            a.save()
            msg = _('The OAuth2 write API for %s was created.') % (a.slug)
            messages.success(request, msg)

            return HttpResponseRedirect(reverse('djmongo_show_apis',
                                                args=(a.database_name,
                                                      a.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            context = {'form': form, 'name': name}
            return render(
                request,
                'djmongo/console/generic/bootstrapform.html',
                context)

    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}
    context = {'name': name,
               'form': WriteAPIOAuth2Form(initial=idata)
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context,
    )


def edit_oauth2_write_api(request, slug):
    a = get_object_or_404(WriteAPIOAuth2, slug=slug)
    name = _("Edit OAuth2 Write API")
    if request.method == 'POST':
        form = WriteAPIOAuth2Form(request.POST, instance=a)
        if form.is_valid():
            a = form.save(commit=False)
            a.created_by = request.user
            a.save()
            msg = _('The OAuth2 API for %s was updated.') % (slug)
            messages.success(request, msg)
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(a.database_name,
                              a.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            context = {'form': form, 'name': name}
            return render(
                request,
                'djmongo/console/generic/bootstrapform.html',
                context)
    # this is a GET
    context = {'name': name,
               'form': WriteAPIOAuth2Form(instance=a)
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context,
    )


def delete_oauth2_write_api(request, slug):
    name = _("Delete OAuth2 Write API")
    if request.method == 'POST':
        form = WriteAPIOAuth2DeleteForm(request.POST)
        if form.is_valid():
            a = WriteAPIOAuth2.objects.get(slug=slug)
            a.delete()
            msg = _('The OAuth2 API for %s was deleted.') % (slug)
            messages.success(request, msg)
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(a.database_name,
                              a.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            context = {'form': form, 'name': name}
            return render(
                request,
                'djmongo/console/generic/bootstrapform.html',
                context)

    # this is a GET
    context = {'name': name,
               'form': WriteAPIOAuth2DeleteForm()
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context,
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4


import binascii
from django.http import HttpResponse
from django.contrib.auth.models import AnonymousUser
from django.contrib.auth import authenticate
from django.contrib.auth import get_user_model


class NoAuthentication(object):
    """
    Authentication handler that always returns
    True, so no authentication is needed, nor
    initiated (`challenge` is missing.)
    """

    def is_authenticated(self, request):
        return True


class HttpBasicAuthentication(object):
    """
    Basic HTTP authenticate. Synopsis:

    Authentication handlers must implement two methods:
     - `is_authenticated`: Will be called when checking for
        authentication. Receives a `request` object, please
        set your `User` object on `request.user`, otherwise
        return False (or something that evaluates to False.)
     - `challenge`: In cases where `is_authenticated` returns
        False, the result of this method will be returned.
        This will usually be a `HttpResponse` object with
        some kind of challenge headers and 401 code on it.
    """

    def __init__(self, auth_func=authenticate, realm='API'):
        self.auth_func = auth_func
        self.realm = realm

    def is_authenticated(self, request):
        auth_string = request.META.get('HTTP_AUTHORIZATION', None)

        if not auth_string:
            return False

        try:
            (authmeth, auth) = auth_string.split(" ", 1)

            if not authmeth.lower() == 'basic':
                return False

            auth = auth.strip().decode('base64')
            (username, password) = auth.split(':', 1)
        except (ValueError, binascii.Error):
            return False

        request.user = self.auth_func(username=username, password=password) \
            or AnonymousUser()

        return request.user not in (False, None, AnonymousUser())

    def authenticate(self, request):
        auth_string = request.META.get('HTTP_AUTHORIZATION', None)

        if not auth_string:
            return AnonymousUser

        try:
            (authmeth, auth) = auth_string.split(" ", 1)

            if not authmeth.lower() == 'basic':
                return AnonymousUser

            auth = auth.strip().decode('base64')
            (username, password) = auth.split(':', 1)
        except (ValueError, binascii.Error):
            return AnonymousUser

        request.user = self.auth_func(username=username, password=password) \
            or AnonymousUser()

        return request.user not in (False, None, AnonymousUser())

    def challenge(self):
        resp = HttpResponse("Authorization Required")
        resp['WWW-Authenticate'] = 'Basic realm="%s"' % self.realm
        resp.status_code = 401
        return resp

    def __repr__(self):
        return u'<HTTPBasic: realm=%s>' % self.realm


class HttpBasicSimple(HttpBasicAuthentication):

    def __init__(self, realm, username, password):
        User = get_user_model()
        self.user = User.objects.get(username=username)
        self.password = password

        super(HttpBasicSimple, self).__init__(auth_func=self.hash, realm=realm)

    def hash(self, username, password):
        if username == self.user.username and password == self.password:
            return self.user



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.conf import settings
from django.http import Http404
from django.shortcuts import render, get_object_or_404
from django.contrib import messages
from django.http import HttpResponseRedirect
from django.urls import reverse
from ..forms import (HTTPAuthReadAPIForm, PublicReadAPIForm,
                     CustomHTTPAuthReadAPIForm, CustomPublicReadAPIForm,
                     IPAuthReadAPIForm, CustomIPAuthReadAPIForm,
                     OAuth2ReadAPIForm, CustomOAuth2ReadAPIForm)

from ..models import (PublicReadAPI, HTTPAuthReadAPI,
                      IPAuthReadAPI, OAuth2ReadAPI,
                      CustomHTTPAuthReadAPI, CustomPublicReadAPI,
                      CustomIPAuthReadAPI, CustomOAuth2ReadAPI)

from django.utils.translation import ugettext_lazy as _

# CREATE API VIEWS -----------------------------


def create_simple_httpauth_read_api(request, database_name, collection_name):
    name = _("Create a Read API with Basic Auth")

    if request.method == 'POST':
        form = HTTPAuthReadAPIForm(request.POST)
        if form.is_valid():
            form.save()
            messages.success(
                request, _("The API was created."))

            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(database_name,
                              collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })
    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}
    context = {'name': name,
               'form': HTTPAuthReadAPIForm(initial=idata)
               }
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def create_simple_ipauth_read_api(request, database_name, collection_name):
    name = _("Create a Read API with IP-Based Auth")

    if request.method == 'POST':
        form = IPReadAPIForm(request.POST)
        if form.is_valid():
            form.save()
            messages.success(
                request, _("The API was created."))

            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(database_name,
                              collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })
    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}
    context = {'name': name,
               'form': IPReadAPIForm(initial=idata)
               }
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def create_simple_oauth2_read_api(request, database_name, collection_name):
    name = _("Create a Read API with OAuth2 Authentication")

    if request.method == 'POST':
        form = OAuth2ReadAPIForm(request.POST)
        if form.is_valid():
            form.save()
            messages.success(
                request, _("The API was created."))

            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(database_name,
                              collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })
    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}
    context = {'name': name,
               'form': OAuth2ReadAPIForm(initial=idata)
               }
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def create_simple_public_read_api(request, database_name, collection_name):
    name = _("Create Simple Read API with No Auth (Public)")

    if request.method == 'POST':
        form = PublicReadAPIForm(request.POST)
        if form.is_valid():
            form.save()
            messages.success(
                request, _("Record created/updated."))

            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(database_name,
                              collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })
    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}
    context = {'name': name,
               'form': PublicReadAPIForm(initial=idata)
               }
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def create_simple_api(request, auth_type, database_name, collection_name):

    if auth_type == "public":
        return create_simple_public_read_api(
            request, database_name, collection_name)

    if auth_type == "httpauth":
        return create_simple_httpauth_read_api(
            request, database_name, collection_name)

    if auth_type == "ipauth":
        return create_simple_ipauth_read_api(
            request, database_name, collection_name)

    if auth_type == "oauth2":
        return create_simple_oauth2_read_api(
            request, database_name, collection_name)

    # else
    raise Http404


def create_custom_public_read_api(
        request,
        database_name=None,
        collection_name=None,
        skip=0,
        limit=getattr(
            settings,
            'MONGO_LIMIT',
            200),
        return_keys=()):
    name = _("Create a Custom Read API with No Auth (Public)")
    if request.method == 'POST':
        form = CustomPublicReadAPIForm(request.POST)
        if form.is_valid():
            form.save()
            messages.success(
                request, _("API created."))
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(database_name,
                              collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })

    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}

    context = {'name': name,
               'form': CustomPublicReadAPIForm(initial=idata)}
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def create_custom_httpauth_read_api(
        request,
        database_name=None,
        collection_name=None,
        skip=0,
        limit=getattr(
            settings,
            'MONGO_LIMIT',
            200),
        return_keys=()):
    name = _("Create a Custom Read API with Basic HTTP Auth")
    if request.method == 'POST':
        form = CustomHTTPAuthReadAPIForm(request.POST)
        if form.is_valid():
            form.save()
            messages.success(
                request, _("API created."))
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(database_name,
                              collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })

    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}

    context = {'name': name,
               'form': CustomHTTPAuthReadAPIForm(initial=idata)}
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def create_custom_ipauth_read_api(
        request,
        database_name=None,
        collection_name=None,
        skip=0,
        limit=getattr(
            settings,
            'MONGO_LIMIT',
            200),
        return_keys=()):
    name = _("Create a Custom Read API with IP-Based Authentication")
    if request.method == 'POST':
        form = CustomIPAuthReadAPIForm(request.POST)
        if form.is_valid():
            form.save()
            messages.success(
                request, _("API created."))
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(database_name,
                              collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name})

    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}

    context = {'name': name,
               'form': CustomIPAuthReadAPIForm(initial=idata)}
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def create_custom_oauth2_read_api(
        request,
        database_name=None,
        collection_name=None,
        skip=0,
        limit=getattr(
            settings,
            'MONGO_LIMIT',
            200),
        return_keys=()):
    name = _("Create a Custom Read API with IP-Based Authentication")
    if request.method == 'POST':
        form = CustomOAuth2ReadAPIForm(request.POST)
        if form.is_valid():
            form.save()
            messages.success(
                request, _("API created."))
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(database_name,
                              collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name})

    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}

    context = {'name': name,
               'form': CustomOAuth2ReadAPIForm(initial=idata)}
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def create_custom_api(request, auth_type, database_name, collection_name,
                      skip=0, limit=getattr(settings, 'MONGO_LIMIT', 200),
                      return_keys=()):
    if auth_type == "public":
        return create_custom_public_read_api(
            request,
            database_name,
            collection_name,
            skip=skip,
            limit=limit,
            return_keys=return_keys)

    if auth_type == "httpauth":
        return create_custom_httpauth_read_api(
            request,
            database_name,
            collection_name,
            skip=skip,
            limit=limit,
            return_keys=return_keys)

    if auth_type == "ipauth":
        return create_custom_ipauth_read_api(
            request,
            database_name,
            collection_name,
            skip=skip,
            limit=limit,
            return_keys=return_keys)

    if auth_type == "oauth2":
        return create_custom_oauth2_read_api(
            request,
            database_name,
            collection_name,
            skip=skip,
            limit=limit,
            return_keys=return_keys)

    # else
    raise Http404


def edit_simple_public_read_api(request, database_name, collection_name, slug):
    name = _("Edit Read API with No Authentication (Public)")
    ss = get_object_or_404(PublicReadAPI, database_name=database_name,
                           collection_name=collection_name, slug=slug)

    if request.method == 'POST':
        form = PublicReadAPIForm(request.POST, instance=ss)
        if form.is_valid():
            ss = form.save()
            messages.success(request, _("Simple Read API Saved."))
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(ss.database_name,
                              ss.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })

    # this is a GET
    context = {'name': name,
               'form': PublicReadAPIForm(instance=ss)
               }
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def edit_simple_httpauth_read_api(
        request,
        database_name,
        collection_name,
        slug):
    name = _("Edit Simple Read API with Basic Auth")
    ss = get_object_or_404(HTTPAuthReadAPI, database_name=database_name,
                           collection_name=collection_name, slug=slug)

    if request.method == 'POST':
        form = HTTPAuthReadAPIForm(request.POST, instance=ss)
        if form.is_valid():
            ss = form.save()
            messages.success(request, _("Simple Read API Saved."))
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(ss.database_name,
                              ss.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })

    # this is a GET
    context = {'name': name,
               'form': HTTPAuthReadAPIForm(instance=ss)
               }
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def edit_simple_ipauth_read_api(
        request,
        database_name,
        collection_name,
        slug):
    name = _("Edit Read API with IP-Based Authentication")
    ss = get_object_or_404(IPAuthReadAPI, database_name=database_name,
                           collection_name=collection_name, slug=slug)

    if request.method == 'POST':
        form = IPAuthReadAPIForm(request.POST, instance=ss)
        if form.is_valid():
            ss = form.save()
            messages.success(request, _("Simple Read API Saved."))
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(ss.database_name,
                              ss.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })

    # this is a GET
    context = {'name': name,
               'form': IPAuthReadAPIForm(instance=ss)
               }
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def edit_simple_oauth2_read_api(
        request,
        database_name,
        collection_name,
        slug):
    name = _("Edit Read API with OAuth2 Authentication")
    ss = get_object_or_404(OAuth2ReadAPI, database_name=database_name,
                           collection_name=collection_name, slug=slug)

    if request.method == 'POST':
        form = OAuth2ReadAPIForm(request.POST, instance=ss)
        if form.is_valid():
            ss = form.save()
            messages.success(request, _("API saved."))
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(ss.database_name,
                              ss.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })

    # this is a GET
    context = {'name': name,
               'form': OAuth2ReadAPIForm(instance=ss)
               }
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def edit_custom_public_read_api(request, database_name, collection_name, slug):
    name = _("Edit Macro Read API with No Authentication (Public)")
    ss = get_object_or_404(CustomPublicReadAPI, database_name=database_name,
                           collection_name=collection_name, slug=slug)

    if request.method == 'POST':
        form = CustomPublicReadAPIForm(request.POST, instance=ss)
        if form.is_valid():
            ss = form.save()
            messages.success(request, _("Simple Read API Saved."))
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(ss.database_name,
                              ss.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })
    # this is a GET
    context = {'name': name,
               'form': CustomPublicReadAPIForm(instance=ss)
               }
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def edit_custom_httpauth_read_api(
        request,
        database_name,
        collection_name,
        slug):
    name = _("Edit Macro Basic HTTP Authentication Read API")
    ss = get_object_or_404(CustomHTTPAuthReadAPI, database_name=database_name,
                           collection_name=collection_name, slug=slug)

    if request.method == 'POST':
        form = CustomHTTPAuthReadAPIForm(request.POST, instance=ss)
        if form.is_valid():
            ss = form.save(commit=False)
            ss.user = request.user
            ss.save()
            messages.success(
                request, _("API edited."))
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(ss.database_name,
                              ss.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })

    # this is a GET
    context = {'name': name,
               'form': CustomHTTPAuthReadAPIForm(instance=ss)
               }
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def edit_custom_ipauth_read_api(
        request,
        database_name,
        collection_name,
        slug):
    print(database_name, collection_name, slug)
    name = _("Edit Macro Read API with IP-Based Authentication")
    ss = get_object_or_404(CustomIPAuthReadAPI, database_name=database_name,
                           collection_name=collection_name, slug=slug)

    if request.method == 'POST':
        form = CustomIPAuthReadAPIForm(request.POST, instance=ss)
        if form.is_valid():
            ss = form.save(commit=False)
            ss.user = request.user
            ss.save()
            messages.success(
                request, _("API updated."))
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(ss.database_name,
                              ss.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })
    # this is a GET
    context = {'name': name,
               'form': CustomIPAuthReadAPIForm(instance=ss)
               }
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def edit_custom_oauth2_read_api(
        request,
        database_name,
        collection_name,
        slug):
    name = _("Edit Macro Read API with OAuth2 Authentication")
    ss = get_object_or_404(CustomOAuth2ReadAPI, database_name=database_name,
                           collection_name=collection_name, slug=slug)

    if request.method == 'POST':
        form = CustomOAuth2ReadAPIForm(request.POST, instance=ss)
        if form.is_valid():
            ss = form.save(commit=False)
            ss.user = request.user
            ss.save()
            messages.success(
                request, _("API updated."))
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(ss.database_name,
                              ss.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })
    # this is a GET
    context = {'name': name,
               'form': CustomOAuth2ReadAPIForm(instance=ss)
               }
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def delete_custom_public_read_api(
        request,
        database_name,
        collection_name,
        slug):
    """Delete CustomPublicReadAPI"""
    ss = get_object_or_404(CustomPublicReadAPI, database_name=database_name,
                           collection_name=collection_name, slug=slug)
    ss.delete()
    messages.success(request, _("API deleted."))
    return HttpResponseRedirect(
        reverse('djmongo_show_apis',
                args=(ss.database_name, ss.collection_name)))


def delete_custom_httpauth_read_api(
        request,
        database_name,
        collection_name,
        slug):
    """Delete CustomHTTPAuthReadAPI"""
    ss = get_object_or_404(CustomHTTPAuthReadAPI, database_name=database_name,
                           collection_name=collection_name, slug=slug)
    ss.delete()
    messages.success(request, _("API deleted."))
    return HttpResponseRedirect(
        reverse('djmongo_show_apis',
                args=(ss.database_name, ss.collection_name)))


def delete_custom_ipauth_read_api(
        request,
        database_name,
        collection_name,
        slug):
    """Delete CustomIPAuthReadAPI"""
    ss = get_object_or_404(CustomIPAuthReadAPI, database_name=database_name,
                           collection_name=collection_name, slug=slug)
    ss.delete()
    messages.success(request, _("API deleted."))
    return HttpResponseRedirect(
        reverse('djmongo_show_apis',
                args=(ss.database_name, ss.collection_name)))


def delete_custom_oauth2_read_api(
        request,
        database_name,
        collection_name,
        slug):
    """Delete CustomOAuth2ReadAPI"""
    ss = get_object_or_404(CustomOAuth2ReadAPI, database_name=database_name,
                           collection_name=collection_name, slug=slug)
    ss.delete()
    messages.success(request, _("API deleted."))
    return HttpResponseRedirect(
        reverse('djmongo_show_apis',
                args=(ss.database_name, ss.collection_name)))


def delete_simple_public_read_api(
        request,
        database_name,
        collection_name,
        slug):
    """Delete Simple PublicReadAPI"""
    ss = get_object_or_404(PublicReadAPI, database_name=database_name,
                           collection_name=collection_name, slug=slug)
    ss.delete()
    messages.success(request, _("API deleted."))
    return HttpResponseRedirect(
        reverse('djmongo_show_apis',
                args=(ss.database_name, ss.collection_name)))


def delete_simple_httpauth_read_api(
        request,
        database_name,
        collection_name,
        slug):
    """Delete Simple PublicReadAPI"""
    ss = get_object_or_404(HTTPAuthReadAPI, database_name=database_name,
                           collection_name=collection_name, slug=slug)
    ss.delete()
    messages.success(request, _("API deleted."))
    return HttpResponseRedirect(
        reverse('djmongo_show_apis',
                args=(ss.database_name, ss.collection_name)))


def delete_simple_ipauth_read_api(
        request,
        database_name,
        collection_name,
        slug):
    """Delete Simple IP Read API"""
    ss = get_object_or_404(IPAuthReadAPI, database_name=database_name,
                           collection_name=collection_name, slug=slug)
    ss.delete()
    messages.success(request, _("API deleted."))
    return HttpResponseRedirect(
        reverse('djmongo_show_apis',
                args=(ss.database_name, ss.collection_name)))


def delete_simple_oauth2_read_api(
        request,
        database_name,
        collection_name,
        slug):
    """Delete Simple PublicReadAPI"""
    ss = get_object_or_404(IPAuthReadAPI, database_name=database_name,
                           collection_name=collection_name, slug=slug)
    ss.delete()
    messages.success(request, _("API deleted."))
    return HttpResponseRedirect(
        reverse('djmongo_show_apis',
                args=(ss.database_name, ss.collection_name)))



# Generated by Django 2.2.4 on 2019-11-05 17:04

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Aggregation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(max_length=100, unique=True)),
                ('pipeline', models.TextField(default='[]', max_length=20480, verbose_name='Pipeline')),
                ('database_name', models.CharField(max_length=256)),
                ('collection_name', models.CharField(max_length=256)),
                ('output_collection_name', models.CharField(help_text='The resulting collection.\n        Do not include $out in your pipeline.', max_length=256)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('execute_now', models.BooleanField(blank=True, default=False)),
                ('scheduled_job_in_process', models.BooleanField(blank=True, default=False)),
                ('scheduled_job_completed_for_today', models.BooleanField(blank=True, default=False)),
                ('scheduled_job_not_ran', models.BooleanField(blank=True, default=True)),
                ('execute_everyday', models.BooleanField(blank=True, default=False)),
                ('execute_sunday', models.BooleanField(blank=True, default=False)),
                ('execute_monday', models.BooleanField(blank=True, default=False)),
                ('execute_tuesday', models.BooleanField(blank=True, default=False)),
                ('execute_wednesday', models.BooleanField(blank=True, default=False)),
                ('execute_thursday', models.BooleanField(blank=True, default=False)),
                ('execute_friday', models.BooleanField(blank=True, default=False)),
                ('execute_saturday', models.BooleanField(blank=True, default=False)),
                ('execute_time_1', models.TimeField(help_text='Use 24 hour time. (e.g. 20:30:00 for 8:30 pm)', verbose_name='Execute Time')),
                ('description', models.TextField(blank=True, default='', max_length=1024)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'Saved Aggregations',
                'ordering': ('-creation_date',),
                'get_latest_by': 'creation_date',
            },
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.contrib.auth.decorators import login_required
from django.conf.urls import url
from ..decorators import httpauth_login_required
from ..decorators import (check_read_httpauth_access,
                          check_public_ok,
                          ipauth_read_verification_required,
                          custom_ipauth_read_verification_required)

from .views.run import (build_keys, simple_search, run_custom_public_read_api_by_slug,
                        run_custom_httpauth_read_api_by_slug,
                        run_custom_ipauth_read_api_by_slug)

from .views.ced import create_simple_api, create_custom_api
from .views.ced import (edit_custom_httpauth_read_api, edit_custom_public_read_api,
                        edit_custom_ipauth_read_api, edit_custom_oauth2_read_api)

from .views.ced import (edit_simple_httpauth_read_api, edit_simple_public_read_api,
                        edit_simple_ipauth_read_api, edit_simple_oauth2_read_api)

from .views.ced import (delete_custom_httpauth_read_api, delete_custom_public_read_api,
                        delete_custom_ipauth_read_api, delete_custom_oauth2_read_api)
from .views.ced import (delete_simple_httpauth_read_api, delete_simple_public_read_api,
                        delete_simple_oauth2_read_api, delete_simple_ipauth_read_api)

urlpatterns = [
    # Run Read APIs ------------------------

    url(r'^api/public/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^.]+).(?P<output_type>[^/]+)$',
        check_public_ok(simple_search), name="djmongo_api_public_simple_search"),

    url(r'^api/httpauth/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^.]+).(?P<output_type>[^/]+)$',
        httpauth_login_required(check_read_httpauth_access(simple_search)),
        name="djmongo_api_httpauth_simple_search"),

    url(r'^api/ipauth/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^.]+).(?P<output_type>[^/]+)$',
        ipauth_read_verification_required(simple_search),
        name="djmongo_api_ipauth_simple_search"),


    url(r'^api/custom/httpauth/(?P<slug>\S+)$',
        httpauth_login_required(run_custom_httpauth_read_api_by_slug),
        name="djmongo_run_custom_httpauth_read_api_by_slug"),

    url(r'^api/custom/public/(?P<slug>\S+)$',
        run_custom_public_read_api_by_slug,
        name="djmongo_run_custom_public_read_api_by_slug"),

    url(r'^api/custom/ipauth/(?P<slug>\S+)$',
        custom_ipauth_read_verification_required(
            run_custom_ipauth_read_api_by_slug),
        name="djmongo_run_custom_ipauth_read_api_by_slug"),

    # CRUD for Managing APIs

    # Create -------------------------------------
    url(r'^simple/(?P<auth_type>[^/]+)/create-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        login_required(create_simple_api),
        name="djmongo_create_simple_api"),

    url(r'^custom/(?P<auth_type>[^/]+)/create-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        login_required(create_custom_api),
        name="djmongo_create_custom_api"),


    # Edit ----------------------------------------
    url(r'^simple/public/edit-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^/]+)$',
        login_required(edit_simple_public_read_api),
        name="djmongo_edit_simple_public_read_api"),

    url(r'^simple/httpauth/edit-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^/]+)$',
        login_required(edit_simple_httpauth_read_api),
        name="djmongo_edit_simple_httpauth_read_api"),

    url(r'^simple/oauth2/edit-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^/]+)$',
        login_required(edit_simple_oauth2_read_api),
        name="djmongo_edit_simple_oauth2_read_api"),

    url(r'^simple/ipauth/edit-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^/]+)$',
        login_required(edit_simple_ipauth_read_api),
        name="djmongo_edit_simple_ipauth_read_api"),


    url(r'^custom/httpauth/edit-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>\S+)$',
        login_required(edit_custom_httpauth_read_api),
        name="djmongo_edit_custom_httpauth_read_api"),

    url(r'^custom/public/edit-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>\S+)$',
        login_required(edit_custom_public_read_api),
        name="djmongo_edit_custom_public_read_api"),

    url(r'^custom/oauth2/edit-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>\S+)$',
        login_required(edit_custom_oauth2_read_api),
        name="djmongo_edit_custom_oauth2_read_api"),

    url(r'^custom/ipauth/edit-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>\S+)$',
        login_required(edit_custom_ipauth_read_api),
        name="djmongo_edit_custom_ipauth_read_api"),




    # Delete --------------------------------------
    url(r'^simple/public/delete-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^/]+)$',
        login_required(delete_simple_public_read_api),
        name="djmongo_delete_simple_public_read_api"),

    url(r'^simple/httpauth/delete-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^/]+)$',
        login_required(delete_simple_httpauth_read_api),
        name="djmongo_delete_simple_httpauth_read_api"),

    url(r'^simple/oauth2/delete-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^/]+)$',
        login_required(delete_simple_oauth2_read_api),
        name="djmongo_delete_simple_oauth2_read_api"),

    url(r'^simple/ipauth/delete-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^/]+)$',
        login_required(delete_simple_ipauth_read_api),
        name="djmongo_delete_simple_ipauth_read_api"),


    url(r'^custom/public/delete-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^/]+)$',
        login_required(delete_custom_public_read_api),
        name="djmongo_delete_custom_public_read_api"),

    url(r'^custom/httpauth/delete-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^/]+)$',
        login_required(delete_custom_httpauth_read_api),
        name="djmongo_delete_custom_httpauth_read_api"),

    url(r'^custom/oauth2/delete-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^/]+)$',
        login_required(delete_custom_oauth2_read_api),
        name="djmongo_delete_custom_oauth2_read_api"),

    url(r'^custom/ipauth/delete-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^/]+)$',
        login_required(delete_custom_ipauth_read_api),
        name="djmongo_delete_custom_ipauth_read_api"),


    # Build keys of a collection.
    url(r'^build-keys/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        login_required(build_keys),
        name="djmongo_build_keys"),


]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4


"""
    Decorator to check for credentials before responding on API requests.
    Response with JSON instead of standard login redirect.
"""
from __future__ import absolute_import
from __future__ import unicode_literals
from django.conf import settings
import base64
from collections import OrderedDict
from functools import update_wrapper
from django.http import HttpResponse
from django.contrib.auth import login, authenticate
from .utils import unauthorized_json_response
from .read.models import HTTPAuthReadAPI, PublicReadAPI, IPAuthReadAPI, CustomIPAuthReadAPI
from .write.models import WriteAPIIP
import shlex
import json


def httpauth_login_required(func):
    """
        Put this decorator before your view to check if the user is logged in
        via httpauth and return a JSON 401 error if he/she is not.
    """

    def wrapper(request, *args, **kwargs):
        user = None
        # get the Basic username and password from the request.
        if 'HTTP_AUTHORIZATION' in request.META:
            auth = request.META['HTTP_AUTHORIZATION'].split()
            if len(auth) == 2:
                if auth[0].lower() == "basic":
                    username, password = base64.b64decode(
                        auth[1]).decode('utf-8').split(':')
                    # print(username, password)
                    user = authenticate(username=username, password=password)

        if not user or not user.is_active:
            return HttpResponse(unauthorized_json_response(),
                                content_type="application/json", status=401)
        login(request, user)
        return func(request, *args, **kwargs)

    return update_wrapper(wrapper, func)


def ip_write_verification_required(func):
    """
        Put this decorator before your view to check if the function is coming from an IP on file
    """

    def wrapper(request, *args, **kwargs):

        slug = kwargs.get('slug', "")
        if not slug:
            return kickout_404("Not found.", content_type="application/json")

        try:
            wip = WriteAPIIP.objects.get(slug=slug)
            ip = get_client_ip(request)
            if ip not in wip.allowable_ips() and "0.0.0.0" not in wip.allowable_ips():
                msg = "The IP %s is not authorized to make the API call." % (
                    ip)
                return kickout_401(msg)

        except WriteAPIIP.DoesNotExist:
            return HttpResponse(unauthorized_json_response(),
                                content_type="application/json")

        return func(request, *args, **kwargs)

    return update_wrapper(wrapper, func)


def ipauth_read_verification_required(func):
    """
        Put this decorator before your view to check if the function is coming from an IP on file
    """

    def wrapper(request, *args, **kwargs):

        slug = kwargs.get('slug', "")
        database_name = kwargs.get('database_name', "")
        collection_name = kwargs.get('collection_name', "")
        if not slug:
            return kickout_404("Not found.", content_type="application/json")

        try:
            rip = IPAuthReadAPI.objects.get(slug=slug, database_name=database_name,
                                            collection_name=collection_name)
            ip = get_client_ip(request)
            if ip not in rip.allowable_ips() and "0.0.0.0" not in rip.allowable_ips():
                msg = "The IP %s is not authorized to make the API call." % (
                    ip)
                return kickout_401(msg)

        except IPAuthReadAPI.DoesNotExist:
            return HttpResponse(unauthorized_json_response(),
                                content_type="application/json")

        return func(request, *args, **kwargs)

    return update_wrapper(wrapper, func)


def custom_ipauth_read_verification_required(func):
    """
        Put this decorator before your view to check if the function is coming from an IP on file
    """

    def wrapper(request, *args, **kwargs):

        slug = kwargs.get('slug', "")
        if not slug:
            return kickout_404("Not found.", content_type="application/json")

        try:
            rip = CustomIPAuthReadAPI.objects.get(slug=slug)
            ip = get_client_ip(request)
            if ip not in rip.allowable_ips() and "0.0.0.0" not in rip.allowable_ips():
                msg = "The IP %s is not authorized to make the API call." % (
                    ip)
                return kickout_401(msg)

        except CustomIPAuthReadAPI.DoesNotExist:
            return HttpResponse(unauthorized_json_response(),
                                content_type="application/json")

        return func(request, *args, **kwargs)

    return update_wrapper(wrapper, func)


def check_public_ok(func):
    """
        Call after login decorator.
    """

    def wrapper(request, *args, **kwargs):
        default_to_open = getattr(settings, 'DEFAULT_TO_OPEN_READ', False)
        database_name = kwargs.get('database_name', "")
        collection_name = kwargs.get('collection_name', "")
        slug = kwargs.get('slug', "")
        if not default_to_open:
            if not database_name or not collection_name:
                return HttpResponse(unauthorized_json_response(),
                                    content_type="application/json")
            try:
                pub_read_api = PublicReadAPI.objects.get(slug=slug,
                                                         database_name=database_name,
                                                         collection_name=collection_name)
            except PublicReadAPI.DoesNotExist:
                return HttpResponse(unauthorized_json_response(),
                                    content_type="application/json")

            # If search keys have been limited...
            if pub_read_api.search_keys:
                search_key_list = shlex.split(pub_read_api.search_keys)
                for k in request.GET.keys():

                    if k not in search_key_list:
                        message = "Search key %s is not allowed." % (k)

                        body = {"code": 400,
                                "message": k,
                                "errors": [message, ]}

                        return HttpResponse(json.dumps(body, indent=4, ),
                                            content_type="application/json")
        return func(request, *args, **kwargs)

    return update_wrapper(wrapper, func)


def check_read_httpauth_access(func):
    """
        Call after login decorator.
    """

    def wrapper(request, *args, **kwargs):
        database_name = kwargs.get('database_name', "")
        collection_name = kwargs.get('collection_name', "")
        slug = kwargs.get('slug', "")
        if not database_name or not collection_name:
            return HttpResponse(unauthorized_json_response(),
                                content_type="application/json")

        try:
            # Check to see if we have a matching record in DB access.
            dac = HTTPAuthReadAPI.objects.get(slug=slug,
                                              database_name=database_name,
                                              collection_name=collection_name)
        except HTTPAuthReadAPI.DoesNotExist:
            return HttpResponse(unauthorized_json_response(),
                                content_type="application/json")

        dac_groups = dac.groups.all()
        user_groups = request.user.groups.all()

        # allowed groups
        in_group = False
        for dg in dac_groups:
            if dg in user_groups:
                in_group = True

        if not in_group:
            message = "NOT-IN-GROUP: You do not have access to this collection. Please see your system administrator."

            body = {"code": 400,
                    "message": message,
                    "errors": [message, ]}
            return HttpResponse(json.dumps(body, indent=4, ),
                                content_type="application/json")

        # If search keys have been limited...
        if dac.search_keys:
            search_key_list = shlex.split(dac.search_keys)
            for k in request.GET.keys():

                if k not in search_key_list:
                    message = "Search key %s  is not allowed." % (k)
                    body = {"code": 400,
                            "message": k,
                            "errors": [message, ]}

                    return HttpResponse(json.dumps(body, indent=4, ),
                                        content_type="application/json")

        return func(request, *args, **kwargs)

    return update_wrapper(wrapper, func)


def get_client_ip(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


def kickout_401(reason, status_code=401):
    response = OrderedDict()
    response["code"] = status_code
    response["status"] = "Authentication error"
    response["errors"] = [reason, ]
    return HttpResponse(json.dumps(response, indent=4),
                        content_type="application/json")


def kickout_400(reason, status_code=400):
    response = OrderedDict()
    response["code"] = status_code
    response["status"] = "Client error"
    response["errors"] = [reason, ]
    return HttpResponse(json.dumps(response, indent=4),
                        content_type="application/json")


def kickout_404(reason, status_code=404):
    response = OrderedDict()
    response["code"] = status_code
    response["status"] = "NOT FOUND"
    response["errors"] = [reason, ]
    return HttpResponse(json.dumps(response, indent=4),
                        content_type="application/json")


def kickout_500(reason, status_code=500):
    response = OrderedDict()
    response["code"] = status_code
    response["status"] = "SERVER SIDE ERROR"
    response["errors"] = [reason, ]
    return HttpResponse(json.dumps(response, indent=4),
                        content_type="application/json")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.contrib.auth.decorators import login_required
from ..decorators import httpauth_login_required
from ..decorators import (check_read_httpauth_access,
                          check_public_ok,
                          ipauth_read_verification_required,
                          custom_ipauth_read_verification_required)
from django.conf.urls import url
from .views.run import (run_public_delete_api,
                        run_httpauth_delete_api,
                        run_ipauth_delete_api)

from .views.ced import (create_delete_api, edit_delete_api, delete_delete_api)

urlpatterns = [

    # DELETE APIs ------------------------

    url(r'^api/public/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^.]+).(?P<output_type>[^/]+)$',
        check_public_ok(run_public_delete_api), name="djmongo_api_public_delete"),

    url(r'^api/httpauth/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^.]+).(?P<output_type>[^/]+)$',
        httpauth_login_required(check_read_httpauth_access(run_httpauth_delete_api)),
        name="djmongo_api_httpauth_delete"),

    url(r'^api/ipauth/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^.]+).(?P<output_type>[^/]+)$',
        ipauth_read_verification_required(run_ipauth_delete_api),
        name="djmongo_api_ipauth_delete"),

    # CRUD for Managing APIs

    # Create -------------------------------------
    url(r'^(?P<auth_type>[^/]+)/create-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        login_required(create_delete_api),
        name="djmongo_delete_api"),

    # Edit ----------------------------------------
    url(r'^(?P<auth_type>[^/]+)/edit-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^/]+)$',
        login_required(edit_delete_api),
        name="djmongo_edit_delete_api"),
    
    # Delete ----------------------------------------
    url(r'^(?P<auth_type>[^/]+)/delete-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^/]+)$',
        login_required(delete_delete_api),
        name="djmongo_delete_delete_api"),


]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

from django.conf import settings
import csv
import sys
from pymongo import MongoClient
from collections import OrderedDict


def bulk_csv_import_mongo(csvfile, database_name, collection_name,
                          delete_collection_before_import=False):
    """return a response_dict  with a list of search results"""
    """method can be insert or update"""

    response_dict = {}
    try:
        mongodb_client_url = getattr(settings, 'MONGODB_CLIENT',
                                     'mongodb://localhost:27017/')
        mc = MongoClient(mongodb_client_url, document_class=OrderedDict)
        db = mc[str(database_name)]
        collection = db[str(collection_name)]

        if delete_collection_before_import:
            collection.remove({})

        # open the csv file.
        # print(csvfile, dir(csvfile))
        csvhandle = csv.reader(open(csvfile.path, 'r'),
                               delimiter=',')

        rowindex = 0
        error_list = []
        success = 0
        for row in csvhandle:

            if rowindex == 0:
                column_headers = row
                cleaned_headers = []
                for c in column_headers:
                    c = c.replace(".", "")
                    c = c.replace("$", "-")
                    c = c.replace(" ", "_")
                    c = c.replace('"', "")
                    cleaned_headers.append(c)
            else:

                record = dict(zip(cleaned_headers, row))
                # if there is no values, skip the key value pair
                kwargs = OrderedDict()

                # Only populate fields that are not blank.
                for k, v in record.items():
                    if v:
                        kwargs[k] = v
                try:

                    collection.insert(kwargs)
                    success += 1
                except:
                    error_message = "Error on row " + \
                        rowindex + ". " + str(sys.exc_info())
                    error_list.append(error_message)

            rowindex += 1

        if error_list:
            response_dict = {}
            response_dict['num_rows_imported'] = rowindex
            response_dict['num_rows_errors'] = len(error_list)
            response_dict['errors'] = error_list
            response_dict['code'] = 400
            response_dict['message'] = "Completed with errors."
        else:

            response_dict = {}
            response_dict['num_rows_imported'] = success
            response_dict['code'] = 200
            response_dict['message'] = "Completed."
        return response_dict

    except:
        # print "Error reading from Mongo"
        # print str(sys.exc_info())
        response_dict['num_results'] = 0
        response_dict['code'] = 400
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response_dict['message'] = str(sys.exc_info())
    return response_dict



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.utils.translation import ugettext_lazy as _
from django import forms
from .models import WriteAPIHTTPAuth, WriteAPIIP, WriteAPIOAuth2
import json
from collections import OrderedDict


class WriteAPIHTTPAuthForm(forms.ModelForm):

    class Meta:
        model = WriteAPIHTTPAuth
        fields = (
            'slug',
            'http_post',
            'http_put',
            'database_name',
            'collection_name',
            'readme_md',
            'groups',
            'json_schema')
    required_css_class = 'required'

    def clean_json_schema(self):
        json_schema = self.cleaned_data["json_schema"]
        try:
            json.loads(json_schema, object_pairs_hook=OrderedDict)
        except ValueError:
            raise forms.ValidationError(_('The JSONSchema was invalid JSON'))
        return json_schema


class WriteAPIHTTPAuthDeleteForm(forms.Form):

    slug = forms.CharField(label=_("Slug"),
                           help_text=_("Retype the slug to confirm deletion."))

    def clean_slug(self):
        slug = self.cleaned_data["slug"]
        try:
            WriteAPIHTTPAuth.objects.get(slug=slug)
        except WriteAPIHTTPAuth.DoesNotExist:
            raise forms.ValidationError(_('There is no API with that slug.'))
        return slug


class WriteAPIIPForm(forms.ModelForm):

    class Meta:
        model = WriteAPIIP
        fields = (
            'slug',
            'http_post',
            'http_put',
            'database_name',
            'collection_name',
            'from_ip',
            'readme_md',
            'json_schema')
    required_css_class = 'required'

    def clean_json_schema(self):
        json_schema = self.cleaned_data["json_schema"]
        try:
            json_schema_load = json.loads(
                json_schema, object_pairs_hook=OrderedDict)
        except ValueError:
            raise forms.ValidationError(_('The JSONSchema was invalid JSON'))
        return json_schema


class WriteAPIIPDeleteForm(forms.Form):

    slug = forms.CharField(label=_("Slug"),
                           help_text=_("Retype the slug to confirm deletion."))

    def clean_slug(self):
        slug = self.cleaned_data["slug"]
        try:
            WriteAPIIP.objects.get(slug=slug)
        except WriteAPIIP.DoesNotExist:
            raise forms.ValidationError(_('There is no API with that slug.'))
        return slug


class WriteAPIOAuth2Form(forms.ModelForm):

    class Meta:
        model = WriteAPIOAuth2
        fields = (
            'slug',
            'http_post',
            'http_put',
            'database_name',
            'collection_name',
            'scopes',
            'readme_md',
            'json_schema')
    required_css_class = 'required'

    def clean_json_schema(self):
        json_schema = self.cleaned_data["json_schema"]
        try:
            json.loads(json_schema, object_pairs_hook=OrderedDict)
        except ValueError:
            raise forms.ValidationError(_('The JSON Schema was invalid JSON.'))
        return json_schema


class WriteAPIOAuth2DeleteForm(forms.Form):

    slug = forms.CharField(label=_("Slug"),
                           help_text=_("Retype the slug to confirm deletion."))

    def clean_slug(self):
        slug = self.cleaned_data["slug"]
        try:
            WriteAPIOAuth2.objects.get(slug=slug)
        except WriteAPIOAuth2.DoesNotExist:
            raise forms.ValidationError(_('There is no API with that slug.'))
        return slug



import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    long_description_content_type="text/markdown",
    name='django-djmongo',
    version='0.7.7.9',
    packages=['djmongo', 'djmongo.console',
              'djmongo.accounts', 'djmongo.accounts.migrations',
              'djmongo.aggregations', 'djmongo.aggregations.migrations',
              'djmongo.write', 'djmongo.write.migrations',
              'djmongo.management.commands',
              'djmongo.management',
              'djmongo.read', 'djmongo.read.views', 'djmongo.read.migrations',
              'djmongo.delete', 'djmongo.delete.views', 'djmongo.delete.migrations',
              'djmongo.dataimport', 'djmongo.dataimport.migrations', ],
    include_package_data=True,
    license='GPL2',
    description='A reusable Django application providing a web interface for MongoDB and a RESTful API Toolkit.',
    long_description=README,
    url='https://github.com/videntity/django-djmongo',
    author='Alan Viars',
    author_email='sales@videntity.com',
    install_requires=[
        'django>=2.2.8', 'pymongo', 'django-widget-tweaks',
        'django-bootstrap-form',
        'django-cors-headers', 'jdt', 'jsonschema',
        'django-localflavor', 'django-markdown-deux', ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

import binascii
from django.http import HttpResponse
from django.contrib.auth.models import User, AnonymousUser


class HTTPAuthBackend(object):

    supports_object_permissions = False
    supports_anonymous_user = False

    def authenticate(self, username=None, password=None):
        try:
            user = User.objects.get(username=username)
        except User.DoesNotExist:
            return None

        if user.check_password(password):
            return user

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None

    def is_authenticated(self, request):
        auth_string = request.META.get('HTTP_AUTHORIZATION', None)
        print("here")

        if not auth_string:
            return False

        try:
            (authmeth, auth) = auth_string.split(" ", 1)

            if not authmeth.lower() == 'basic':
                return False

            auth = auth.strip().decode('base64')
            (username, password) = auth.split(':', 1)
        except (ValueError, binascii.Error):
            return False

        request.user = self.auth_func(username=username, password=password) \
            or AnonymousUser()

        return request.user not in (False, None, AnonymousUser())

    def challenge(self):
        resp = HttpResponse("Authorization Required")
        resp['WWW-Authenticate'] = 'Basic realm="%s"' % self.realm
        resp.status_code = 401
        return resp

    def __repr__(self):
        return u'<HTTPBasic: realm=%s>' % self.realm


class BasicBackend:

    supports_object_permissions = False
    supports_anonymous_user = False

    def authenticate(self, username=None, password=None):
        # print "Basic backend", username
        try:
            user = User.objects.get(username=username)
        except User.DoesNotExist:
            return None

        if user.check_password(password):
            return user

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None


class EmailBackend(BasicBackend):

    supports_object_permissions = False
    supports_anonymous_user = False

    def authenticate(self, username=None, password=None):

        try:
            user = User.objects.get(email=username)
        except User.DoesNotExist:
            return None

        if user.check_password(password):
            return user


class MobilePINBackend(BasicBackend):

    supports_object_permissions = False
    supports_anonymous_user = False

    def authenticate(self, username=None, password=None):
        # We have a non-email address username we should try username
        try:
            user = User.objects.get(mobile_phone_number=username)
        except User.DoesNotExist:
            return None

        if str(user.pin) == str(password):
            return user
        else:
            return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.db import models
import os
import uuid
import json
from .utils import bulk_csv_import_mongo


def update_import_filename(instance, filename):
    path = "imports/"
    format = instance.database_name + "-" + \
        instance.collection_name  + "-" + \
        str(uuid.uuid4())[0:5] + "-" + filename
    return os.path.join(path, format)


INPUT_CHOICES = (("csv", "Comma Separated Value (.csv)"),)


class DataImport(models.Model):

    file1 = models.FileField(upload_to=update_import_filename,
                             verbose_name="File to be Imported")
    input_format = models.CharField(max_length=3,
                                    choices=INPUT_CHOICES,
                                    default="csv")

    delete_collection_before_import = models.BooleanField(default=False)

    status = models.CharField(max_length=10,
                              default="New",
                              verbose_name="Status", editable=False)

    response = models.TextField(max_length=2048, default="", blank=True,
                                verbose_name="Response")

    database_name = models.CharField(max_length=256)

    collection_name = models.CharField(max_length=256)

    creation_date = models.DateField(auto_now_add=True)

    class Meta:
        get_latest_by = "creation_date"
        ordering = ('-creation_date',)

    def __str__(self):
        return "%s/%s" % (self.database_name, self.collection_name)

    def save(self, commit=True, **kwargs):
        self.status = "Processing"

        # Generate the slug if the record it was not already defined.
        super(DataImport, self).save(**kwargs)
        # process the file
        result = bulk_csv_import_mongo(self.file1,
                                       self.database_name,
                                       self.collection_name,
                                       self.delete_collection_before_import)

        print(result)
        # report results
        self.response = json.dumps(result, indent=4)
        self.status = "Complete"
        # re-write db after complete
        if commit:
            super(DataImport, self).save(**kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

from django.conf.urls import url
from .views import (api_test_credentials, simple_logout, simple_login,
                    user_password, api_user_create, api_read_user,
                    api_delete_user, api_user_update, user_create,
                    user_update)
from .decorators import json_login_required
from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_exempt

urlpatterns = [

    # API Calls (these return JSON). ------------------------------------------
    url(r'api/test-credentials', api_test_credentials,
        name='api-test-credentials'),
    url(r'api/user/create', csrf_exempt(json_login_required(api_user_create)),
        name='djmongo_api_user_create'),
    url(r'api/user/update', api_user_update, name='djmongo_api_user_update'),
    url(r'api/user/read/(?P<email>[^/]+)',
        api_read_user, name='djmongo_api_read_user'),
    url(r'api/user/delete/(?P<email>[^/]+)',
        api_delete_user,
        name='djmongo_api_delete_user'),

    # Web Views that require csrf token -------------------------------
    url(r'user/create', login_required(user_create),
        name='djmongo_user_create'),
    url(r'user/update', login_required(user_update),
        name='djmongo_user_update'),
    url(r'user/password',
        login_required(user_password),
        name='djmongo_user_password'),
    # Login.Logout of the web interface
    url(r'^logout$', simple_logout, name="djmongo_logout"),
    url(r'^login$', simple_login, name="djmongo_login"),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

from django import forms
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _


class LoginForm(forms.Form):
    username = forms.CharField()
    password = forms.CharField(widget=forms.PasswordInput,
                               label=_("Password"))
    required_css_class = 'required'


class UserCreationForm(forms.ModelForm):
    """
A form that creates a user, with no privileges, from the given username and
password.
"""
    error_messages = {
        'duplicate_username': _("A user with that username already exists."),
        'duplicate_email': _("A user with that email already exists."),
        'password_mismatch': _("The two password fields didn't match."),
    }
    username = forms.RegexField(
        label=_("Username"),
        max_length=30,
        regex=r'^[\w.@+-]+$',
        help_text=_(
            "Required. 30 characters or fewer. Letters, digits and "
            "@/./+/-/_ only."),
        error_messages={
            'invalid': _(
                "This value may contain only letters, numbers and "
                "@/./+/-/_ characters.")})

    email = forms.EmailField(label=_("Email"))

    password1 = forms.CharField(label=_("Password"),
                                widget=forms.PasswordInput)
    password2 = forms.CharField(
        label=_("Password confirmation"),
        widget=forms.PasswordInput,
        help_text=_("Enter the same password as above, for verification."))

    class Meta:
        model = get_user_model()
        fields = ("first_name", "last_name", "username", "email",)

    def clean_username(self):
        # Since User.username is unique, this check is redundant,
        # but it sets a nicer error message than the ORM. See #13147.
        username = self.cleaned_data["username"]
        try:
            User = get_user_model()
            User._default_manager.get(username=username)
        except User.DoesNotExist:
            return username
        raise forms.ValidationError(
            self.error_messages['duplicate_username'],
            code='duplicate_username',
        )
        return username

    def clean_email(self):
        email = self.cleaned_data["email"]
        try:
            User = get_user_model()
            User._default_manager.get(email=email)
        except User.DoesNotExist:
            return email
        raise forms.ValidationError(
            self.error_messages['duplicate_email'],
            code='duplicate_email',
        )
        return email

    def clean_password2(self):
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError(
                self.error_messages['password_mismatch'],
                code='password_mismatch',
            )
        return password2

    def save(self, commit=True):
        user = super(UserCreationForm, self).save(commit=False)
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
        return user


class UserUpdateForm(forms.ModelForm):
    """
A form that creates a user, with no privileges, from the given username and
password.
"""
    error_messages = {
        'duplicate_username': _("A user with that username already exists."),
        'duplicate_email': _("A user with that email already exists."),
        'password_mismatch': _("The two password fields didn't match."),
    }
    username = forms.RegexField(
        label=_("Username"),
        max_length=30,
        regex=r'^[\w.@+-]+$',
        help_text=_(
            "Required. 30 characters or fewer. Letters, digits and "
            "@/./+/-/_ only."),
        error_messages={
            'invalid': _(
                "This value may contain only letters, numbers and "
                "@/./+/-/_ characters.")})

    email = forms.EmailField(label=_("Email"))

    password1 = forms.CharField(label=_("Password"),
                                widget=forms.PasswordInput)
    password2 = forms.CharField(
        label=_("Password confirmation"),
        widget=forms.PasswordInput,
        help_text=_("Enter the same password as above, for verification."))

    class Meta:
        model = get_user_model()
        fields = ("first_name", "last_name", "username", "email")

    def clean_password2(self):
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError(
                self.error_messages['password_mismatch'],
                code='password_mismatch',
            )
        return password2

    def save(self, commit=True):
        user = super(UserUpdateForm, self).save(commit=False)
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
        return user


class APIUserCreationForm(UserCreationForm):

    def save(self, commit=True):
        user = super(UserCreationForm, self).save(commit=False)
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()

        userdict = {'username': str(user.username),
                    'email': str(user.email),
                    'first_name': str(user.first_name),
                    'last_name': str(user.last_name),
                    }

        result = {"code": 200,
                  "message": "User created successfully",
                  "user": userdict
                  }
        return result


class APIUserUpdateForm(UserUpdateForm):

    def save(self, commit=True):
        user = super(UserUpdateForm, self).save(commit=False)
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()

        userdict = {'username': str(user.username),
                    'email': str(user.email),
                    'first_name': str(user.first_name),
                    'last_name': str(user.last_name),
                    }

        result = {"code": 200,
                  "message": "User updated successfully",
                  "user": userdict
                  }
        return result



from django.db import models
from django.contrib.auth.models import Group
from django.utils.translation import ugettext_lazy as _
from django.urls import reverse


class HTTPAuthDeleteAPI(models.Model):

    database_name = models.CharField(max_length=256)
    collection_name = models.CharField(max_length=256)
    slug = models.SlugField(max_length=100,
                            help_text="Give your API a unique name")
    search_keys = models.TextField(max_length=4096, default="", blank=True,
                                   help_text="""The default, blank, returns
                                                all keys. Providing a list of
                                                keys, separated by whitespace,
                                                limits the API search to only
                                                these keys.""")
    groups = models.ManyToManyField(
        Group, blank=True, related_name="djmongo_http_auth_delete_api")
    readme_md = models.TextField(max_length=4096, default="", blank=True)
    creation_date = models.DateField(auto_now_add=True)

    class Meta:
        # get_latest_by = "creation_date"
        # ordering = ('-creation_date',)
        unique_together = (('database_name', 'collection_name', 'slug'), )

    def __str__(self):
        return "%s/%s" % (self.database_name, self.collection_name)

    def url(self):
        return reverse('djmongo_api_httpauth_delete',
                       args=(self.database_name, self.collection_name, self.slug))

    def http_methods(self):
        return ['DELETE', 'GET', ]

    def auth_method(self):
        return 'httpauth'


class PublicDeleteAPI(models.Model):

    database_name = models.CharField(max_length=256)
    collection_name = models.CharField(max_length=256)
    slug = models.SlugField(max_length=100,
                            help_text="Give your API a unique name")
    search_keys = models.TextField(max_length=4096, default="", blank=True,
                                   help_text="""The default, blank, returns
                                                all keys. Providing a list of
                                                keys, separated by whitespace,
                                                limits the API search to only
                                                these keys.""")
    readme_md = models.TextField(max_length=4096, default="", blank=True)
    creation_date = models.DateField(auto_now_add=True)

    class Meta:
        # get_latest_by = "creation_date"
        # ordering = ('-creation_date',)
        unique_together = (('database_name', 'collection_name', 'slug'), )

    def __str__(self):
        return "%s/%s/%s" % (self.database_name, self.collection_name, self.slug)

    def url(self):
        return reverse('djmongo_api_public_delete',
                       args=(self.database_name, self.collection_name, self.slug))

    def http_methods(self):
        return ['DELETE', 'GET', ]

    def auth_method(self):
        return 'public'


class IPAuthDeleteAPI(models.Model):

    database_name = models.CharField(max_length=256)
    collection_name = models.CharField(max_length=256)
    slug = models.SlugField(max_length=100,
                            help_text="Give your API a unique name")
    from_ip = models.TextField(max_length=2048, default="127.0.0.1",
                               verbose_name=_("From IPs"),
                               help_text=_("Only accept requests from a IP in "
                                           "this list separated by whitespace "
                                           ". 0.0.0.0 means all."))
    search_keys = models.TextField(max_length=4096, default="", blank=True,
                                   help_text="""The default, blank, returns
                                                all keys. Providing a list of
                                                keys, separated by whitespace,
                                                limits the API search to only
                                                these keys.""")
    readme_md = models.TextField(max_length=4096, default="", blank=True)
    creation_date = models.DateField(auto_now_add=True)

    class Meta:
        # get_latest_by = "creation_date"
        # ordering = ('-creation_date',)
        unique_together = (('database_name', 'collection_name', 'slug'), )

    def allowable_ips(self):
        allowable_ips = self.from_ip.split(" ")
        return allowable_ips

    def __str__(self):
        return "%s/%s/%s" % (self.database_name, self.collection_name, self.slug)

    def url(self):
        return reverse('djmongo_api_ipauth_delete',
                       args=(self.database_name, self.collection_name, self.slug, 'json'))

    def http_methods(self):
        return ['DELETE', 'GET', ]

    def auth_method(self):
        return 'ipauth'


class OAuth2DeleteAPI(models.Model):

    database_name = models.CharField(max_length=256)
    collection_name = models.CharField(max_length=256)
    slug = models.SlugField(max_length=100,
                            help_text="Give your API a unique name")
    scopes = models.CharField(max_length=1024, default="*", blank=True,
                              help_text="Space delimited list of scopes required. * means no scope is required.")
    search_keys = models.TextField(max_length=4096, default="", blank=True,
                                   help_text="""The default, blank, returns
                                                all keys. Providing a list of
                                                keys, separated by whitespace,
                                                limits the API search to only
                                                these keys.""")
    readme_md = models.TextField(max_length=4096, default="", blank=True)
    creation_date = models.DateField(auto_now_add=True)

    class Meta:
        # get_latest_by = "creation_date"
        # ordering = ('-creation_date',)
        unique_together = (('database_name', 'collection_name', 'slug'), )

    def __str__(self):
        return "%s/%s/%s" % (self.database_name, self.collection_name, self.slug)

    def url(self):
        return reverse('djmongo_api_oauth2_delete',
                       args=(self.database_name, self.collection_name, self.slug))

    def http_methods(self):
        return ['DELETE', 'GET']

    def auth_method(self):
        return 'oauth2'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

from django.contrib.auth.decorators import login_required
from django.conf.urls import url
from .views import (browse_saved_aggregations, create_saved_aggregation,
                    delete_saved_aggregation_by_slug,
                    edit_saved_aggregation_by_slug, run_aggregation_by_slug)


urlpatterns = [

    url(r'^run-aggregation/(?P<slug>\S+)$',
        run_aggregation_by_slug,
        name="djmongo_run_aggregation_by_slug"),

    url(r'^edit-saved-aggregation/(?P<slug>\S+)$',
        login_required(edit_saved_aggregation_by_slug),
        name="djmongo_edit_saved_aggregation_by_slug"),

    url(r'^delete-saved-aggregation/(?P<slug>\S+)$',
        login_required(delete_saved_aggregation_by_slug),
        name="djmongo_delete_saved_aggregation_by_slug"),

    url(r'^browse-saved-aggregations/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        login_required(browse_saved_aggregations),
        name="djmongo_browse_saved_aggregations_w_params"),

    url(r'^create-saved-aggregation/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        login_required(create_saved_aggregation),
        name="djmongo_create_saved_aggregation_w_params"),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.db import models
from django.contrib.auth.models import Group
from django.conf import settings
import json
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import python_2_unicode_compatible
from django.urls import reverse
from collections import OrderedDict
from django.contrib.auth import get_user_model


@python_2_unicode_compatible
class WriteAPIHTTPAuth(models.Model):
    created_by = models.ForeignKey(
        get_user_model(), blank=True, null=True, on_delete=models.CASCADE)
    groups = models.ManyToManyField(
        Group,
        blank=True,
        related_name="djmongo_write_httpauth_groups",
        help_text=_(
            "Use ctrl to select multiple "
            "groups. If no groups are "
            "selected blank, any user may "
            "access the API."))
    slug = models.SlugField(max_length=100, unique=True, help_text=_(
        "The slug is the unique part of the URL for your API."))

    http_post = models.BooleanField(default=True, blank=True)
    http_put = models.BooleanField(default=True, blank=True)
    database_name = models.CharField(max_length=100)
    collection_name = models.CharField(max_length=100)
    json_schema = models.TextField(
        max_length=20480,
        default="{}",
        verbose_name="JSON Schema",
        help_text="""Default "{}", means no JSON Schema.""")
    readme_md = models.TextField(max_length=4096, default="", blank=True)
    creation_date = models.DateField(auto_now_add=True)

    class Meta:
        get_latest_by = "creation_date"
        ordering = ('-creation_date',)
        verbose_name_plural = _("Write APIs with HTTPAuth")
        verbose_name = _("Write API with HTTPAuth")

    def url(self):
        return reverse('djmongo_api_write_to_collection_with_httpauth', args=(self.slug,))

    def __str__(self):
        return "%s" % (self.slug)

    def allgroups(self):
        groups = []
        for g in self.groups.all():
            groups.append(g.name)
        return json.dumps(groups)

    def http_methods(self):
        l = []
        if self.http_post:
            l.append("POST")
        if self.http_put:
            l.append("PUT")
        return l

    def auth_method(self):
        return 'httpauth'

    def http_get_response(self):
        od = OrderedDict()
        od['http_methods'] = self.http_methods()
        od['auth_method'] = self.auth_method()
        od['json_schema'] = self.json_schema
        od['readme'] = self.readme_md
        return od


@python_2_unicode_compatible
class WriteAPIIP(models.Model):
    created_by = models.ForeignKey(
        get_user_model(), blank=True, null=True, on_delete=models.CASCADE)
    slug = models.SlugField(max_length=100, unique=True)
    http_post = models.BooleanField(default=True, blank=True)
    http_put = models.BooleanField(default=True, blank=True)
    database_name = models.CharField(max_length=100)
    collection_name = models.CharField(max_length=100)
    json_schema = models.TextField(
        max_length=20480,
        default="{}",
        verbose_name="JSON Schema",
        help_text="""Default "{}", means no JSON Schema.""")
    from_ip = models.TextField(max_length=2048, default="127.0.0.1",
                               verbose_name=_("From IPs"),
                               help_text=_("Only accept requests from a IP in "
                                           "this list separated by whitespace "
                                           ". 0.0.0.0 means all."))
    readme_md = models.TextField(max_length=4096, default="", blank=True)
    creation_date = models.DateField(auto_now_add=True)

    class Meta:
        get_latest_by = "creation_date"
        ordering = ('-creation_date',)
        verbose_name_plural = _("Write APIs with IP Address Auth")
        verbose_name = _("Write API with IP Address Auth")

    def allowable_ips(self):
        allowable_ips = self.from_ip.split(" ")
        return allowable_ips

    def __str__(self):
        return "%s" % (self.slug)

    def url(self):
        return reverse('djmongo_api_write_to_collection_with_ip', args=(self.slug,))

    def http_methods(self):
        l = []
        if self.http_post:
            l.append("POST")
        if self.http_put:
            l.append("PUT")
        return l

    def auth_method(self):
        return 'ip'

    def http_get_response(self):
        od = OrderedDict()
        od['http_methods'] = self.http_methods()
        od['auth_method'] = self.auth_method()
        od['json_schema'] = self.json_schema
        od['readme'] = self.readme_md
        return od


@python_2_unicode_compatible
class WriteAPIOAuth2(models.Model):

    created_by = models.ForeignKey(
        get_user_model(), blank=True, null=True, on_delete=models.CASCADE)
    scopes = models.CharField(max_length=1024, default="*", blank=True,
                              help_text="Space delimited list of scopes required. * means no scope is required.")
    http_post = models.BooleanField(default=True, blank=True)
    http_put = models.BooleanField(default=True, blank=True)
    slug = models.SlugField(max_length=100, unique=True)
    database_name = models.CharField(max_length=100)
    collection_name = models.CharField(max_length=100)
    json_schema = models.TextField(
        max_length=20480,
        default="{}",
        verbose_name="JSON Schema",
        help_text="""Default "{}", means no JSON Schema.""")
    readme_md = models.TextField(max_length=4096, default="", blank=True)
    creation_date = models.DateField(auto_now_add=True)

    class Meta:
        get_latest_by = "creation_date"
        ordering = ('-creation_date',)
        verbose_name_plural = "Write APIs with OAuth2 Auth"
        verbose_name = "Write API with OAuth2 Auth"

    def __str__(self):
        return "%s" % (self.slug)

    def http_methods(self):
        l = []
        if self.http_post:
            l.append("POST")
        if self.http_put:
            l.append("PUT")
        return l

    def auth_method(self):
        return 'oauth2'

    def http_get_response(self):
        od = OrderedDict()
        od['http_methods'] = self.http_methods()
        od['auth_method'] = self.auth_method()
        od['json_schema'] = self.json_schema
        od['readme'] = self.readme_md
        return od



from django.contrib import admin
from .models import DataImport


class DataImport2Admin(admin.ModelAdmin):
    list_display = ('database_name', 'collection_name',
                    'status', 'creation_date')


admin.site.register(DataImport, DataImport2Admin)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

from django.db import models
from django.conf import settings
import datetime
from django.contrib.auth import get_user_model

PERMISSION_CHOICES = (('db-all', 'All MongoDB'),
                      ('db-write', 'Write MongoDB'),
                      ('db-read', 'Read MongoDB'),
                      ('create-other-users', 'create-other-users'),
                      ('create-any-socialgraph', 'create-any-socialgraph'),
                      ('delete-any-socialgraph', 'delete-any-socialgraph'),
                      )


class Permission(models.Model):
    user = models.ForeignKey(get_user_model(), on_delete=models.CASCADE)
    permission_name = models.CharField(max_length=256,
                                       choices=PERMISSION_CHOICES)

    def __unicode__(self):
        return '%s has the %s permission.' % (
            self.user.email, self.permission_name)

    class Meta:
        unique_together = (("user", "permission_name"),)


class SocialGraph(models.Model):

    grantor = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name="grantor", on_delete=models.CASCADE)
    grantee = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name="grantee", on_delete=models.CASCADE)
    created_on = models.DateField(default=datetime.date.today)

    def __unicode__(self):
        return "%s --> %s since %s" % (self.grantor.username,
                                       self.grantee.username,
                                       self.created_on)

    class Meta:
        unique_together = (("grantor", "grantee"),)
        ordering = ('-created_on',)
        get_latest_by = "created_on"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

from django.conf.urls import include, url
from django.views.generic.base import TemplateView


urlpatterns = [

    url(r'^import/', include('djmongo.dataimport.urls')),
    url(r'^console/', include('djmongo.console.urls')),
    url(r'^read/', include('djmongo.read.oauth2_urls')),
    url(r'^write/', include('djmongo.write.oauth2_urls')),
    url(r'^accounts/', include('djmongo.accounts.urls')),
    url(r'^aggregations/', include('djmongo.aggregations.urls')),
    url(r'^$', TemplateView.as_view(template_name='djmongo/console/splash.html'),
        name='djmongo_splash'),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

from django.forms import ModelForm
from .models import DataImport


class DataImportForm(ModelForm):

    class Meta:
        model = DataImport
        fields = ('file1', 'delete_collection_before_import',
                  'input_format', 'database_name', 'collection_name')
    required_css_class = 'required'
